from poap.controller import SerialController, ThreadController, BasicWorkerThread, SimTeamController
from sot_sync_strategies import *
from sot_async_strategies import *
from rbf_interpolant import *
from sampling_methods import *
from rbf_surfaces import *
from rs_capped import *
from numpy.matlib import repmat
from ensemble_surrogate import *
from poly_regression import *
from utils import *

class Ackley:
    def __init__(self, dim=10):
        self.xlow = -15 * np.ones(dim)
        self.xup = 20 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Ackley function \n" +\
                             "Global optimum: f(0,0,...,0) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        n = float(len(x))
        return -20.0 * np.exp(-0.2*np.sqrt(sum(x**2)/n)) - \
            np.exp(sum(np.cos(2.0*np.pi*x))/n)


class Rastrigin:
    def __init__(self, dim=10):
        self.xlow = -4 * np.ones(dim)
        self.xup = 5 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Rastrigin function \n" +\
                             "Global optimum: f(0,0,...,0) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return np.sum(x**2 - np.cos(2 * np.pi * x))


class Griewank:
    def __init__(self, dim=10):
        self.xlow = -500 * np.ones(dim)
        self.xup = 700 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Griewank function \n" +\
                             "Global optimum: f(0,0,...,0) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        total = 1
        for i, y in enumerate(x):
            total *= np.cos(y / np.sqrt(i+1))
        return 1.0 / 4000.0 * sum([y**2 for y in x]) - total + 1


class Michalewicz:
    def __init__(self, dim=10):
        self.xlow = np.zeros(dim)
        self.xup = np.pi * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Michalewicz function \n" +\
                             "Global optimum: ??"
        self.min = np.NaN
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return -np.sum(np.sin(x) * (np.sin(((1 + np.arange(self.dim)) * x ** 2)/np.pi)) ** 20)


class Keane:
    def __init__(self, dim=30):
        self.xlow = 1.0 * np.ones(dim)
        self.xup = 10.0 * np.ones(dim)
        self.dim = dim
        self.min = -0.835
        self.integer = []
        self.continuous = np.arange(0, dim)
        self.info = str(dim)+"-dimensional Keane bump function \n" +\
                             "Global optimum: -0.835 for large n"

    def objfunction(self, x):
        d = len(x)
        y1 = 0.0
        y2 = 1.0
        y3 = 0.0

        for i in range(d):
            y1 += (np.cos(x[i]) ** 4)

        for i in range(d):
            y2 *= (np.cos(x[i]) ** 2)

        for i in range(d):
            y3 += (i+1) * (x[i] ** 2)

        y = (y1 - 2.0 * y2) / np.sqrt(y3)

        return - np.abs(y)


class Rosenbrock:
    def __init__(self, dim=10):
        self.xlow = -2.048 * np.ones(dim)
        self.xup = 2.048 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Rosenbrock function \n" +\
                             "Global optimum: f(1,1,...,1) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        total = 0
        for i in range(len(x) - 1):
            total += 100 * (x[i]**2 - x[i+1])**2 + (x[i] - 1)**2
        return total


class Schwefel:
    def __init__(self, dim=10):
        self.xlow = -512 * np.ones(dim)
        self.xup = 512 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Schwefel function \n" +\
                             "Global optimum: f(420.968746,...,420.968746) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return 418.9829 * self.dim - \
            sum([y * np.sin(np.sqrt(abs(y))) for y in x])


class Sphere:
    def __init__(self, dim=10):
        self.xlow = -15 * np.ones(dim)
        self.xup = 20 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Sphere function \n" +\
                             "Global optimum: f(0,0,...,0) = 0"
        self.min = 0
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        """Evaluate the Sphere function  at x

        :param x: Data point
        :return: Value at x
        """
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return sum(x**2)


class Himmelblau:
    def __init__(self, dim=30):
        self.xlow = -100 * np.ones(dim)
        self.xup = 100 * np.ones(dim)
        self.dim = dim
        self.min = -78.3323
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return (1.0/self.dim) * np.sum(x**4 - 16*(x**2) + 5*x)


class Schwefel2_26:
    def __init__(self, dim=30):
        self.xlow = -500 * np.ones(dim)
        self.xup = 500 * np.ones(dim)
        self.dim = dim
        self.min = -12569.48
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return - np.dot(x, np.sin(np.sqrt(np.abs(x))))


class Exponential:
    def __init__(self, dim=10):
        self.xlow = np.zeros(dim)
        self.xup = 5 * np.ones(dim)
        self.dim = dim
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        return np.exp(np.sum(x))


class RotatedSphere:
    def __init__(self, dim=30):
        self.xlow = -100 * np.ones(dim)
        self.xup = 100 * np.ones(dim)
        self.dim = dim
        self.integer = []
        self.continuous = np.arange(0, dim)
        mat = np.random.randn(dim, dim)
        Q, R = np.linalg.qr(mat)
        self.rotation = Q
        self.optimum = np.zeros(dim)

    def objfunction(self, xx):
        if len(xx) != self.dim:
            raise ValueError('Dimension mismatch')
        x = self.optimum + np.dot(self.rotation, (xx - self.optimum))
        return sum(x ** 2)


class Schoen:
    def __init__(self, dim=30, k=None, fi=None, z=None):
        self.xlow = np.zeros(dim)
        self.xup = np.ones(dim)
        self.dim = dim
        self.integer = []
        self.continuous = np.arange(0, dim)
        self.k = k
        if self.k is None:
            self.k = 50
        self.fi = fi
        if self.fi is None:
            self.fi = -np.abs(np.random.randn(self.k, 1).ravel())
        self.z = z
        if self.z is None:
            self.z = np.random.rand(self.k, self.dim)

        self.min = np.amin(self.fi)
        self.optimum = self.z[np.argmin(self.fi), :]

    def objfunction(self, xx):
        if len(xx) != self.dim:
            raise ValueError('Dimension mismatch')
        x = np.atleast_2d(xx)
        prodmat = repmat(np.sum((self.z - x) ** 2, 1), self.k, 1).T
        np.fill_diagonal(prodmat, 1)
        prodvec = np.prod(prodmat, axis=0)
        return np.dot(prodvec, self.fi) / np.sum(prodvec)


def test_ensamble():
    dim = 10
    maxeval = 500
    data = Ackley(dim=dim)

    bounds = np.vstack((data.xlow.T, data.xup.T)).T
    basisp = basis_TD(dim, 2)
    surf = PolyRegression(bounds, basisp)

    models = [
        RBFInterpolant(surftype=CubicRBFSurface, maxp=maxeval),
        surf
    ]
    exp_design = LatinHypercube(dim=data.dim, npts=50)
    X = from_unit_box(exp_design.generate_points(), data)
    response_surface = EnsembleSurrogate(models, maxeval)
    for i in range(exp_design.npts):
        response_surface.add_point(X[i, :], data.objfunction(X[i, :]))

    for i in range(500):
        if response_surface.nump == 50:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
        if response_surface.nump == 100:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
        if response_surface.nump == 200:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
        if response_surface.nump == 300:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
        if response_surface.nump == 400:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
        if response_surface.nump == 500:
            print response_surface.nump
            temp = response_surface.eval(np.zeros(dim,))
            die
        randpt = from_unit_box(np.random.rand(dim,), data)
        response_surface.add_point(randpt, data.objfunction(randpt))

if __name__ == "__main__":
    nruns = 5
    dim = 30
    maxeval = 500
    nthreads = 1

    datavec = [Ackley(dim=dim), Rastrigin(dim=dim), Griewank(dim=dim), Keane(dim=dim), Michalewicz(dim=dim)]
    for data in datavec:
        print "\n" + data.__class__.__name__

        vals = np.zeros(nruns,)
        for ii in range(nruns):
            controller = SerialController(data.objfunction)

            nsamples = nthreads

            controller.strategy = \
                SyncStrategyNoConstraints(
                    worker_id=0, data=data,
                    maxeval=maxeval, nsamples=nsamples,
                    exp_design=SymmetricLatinHypercube(dim=data.dim, npts=2*(data.dim+1)),
                    response_surface=RBFInterpolant(surftype=CubicRBFSurface, maxp=maxeval),
                    sampling_method=CandidateDYCORS(data=data, numcand=100*data.dim))

            # Run the optimization strategy
            result = controller.run()

            print(result.value)
            vals[ii] = result.value
        print np.min(vals), np.max(vals), np.median(vals), np.mean(vals), np.std(vals)/np.sqrt(nruns)





"""
.. module:: kriging_interpolant
   :synopsis: kriging model interpolation
.. moduleauthor:: Yi Shen <ys623@cornell.edu>

:Module: kriging_interpolant
:Author: Yi Shen <ys623@cornell.edu>
"""

import numpy as np
from pyKriging.krige import kriging


class KrigingInterpolant:
    """Compute and evaluate Kriging interpolant.

    :ivar nump: Current number of points
    :ivar maxp: Initial maximum number of points (can grow)
    :ivar x: Interpolation points
    :ivar fx: Function values at interpolation points
    :ivar k: Kriging model instance
    :ivar updated: Flag that indicates whether a Kriging
        model is up-to-date or not
    """

    def __init__(self, maxp=100):
        self.nump = 0
        self.maxp = maxp
        self.x = None
        self.fx = None
        self.dim = None
        self.k = None
        self.updated = False

    def reset(self):
        """Reset the Kriging interpolant
        """
        self.nump = 0
        self.x = None
        self.fx = None
        self.updated = False

    def _alloc(self, dim):
        """Allocate storage for x, fx and rhs.

        :param dim: Number of dimensions
        """
        maxp = self.maxp
        self.dim = dim
        self.x = np.zeros((maxp, dim))
        self.fx = np.zeros((maxp, 1))

    def _realloc(self, dim, extra=1):
        """Expand allocation to accommodate more points (if needed)

        :param dim: Number of dimensions
        :param extra: Number of additional points to accommodate
        """
        if self.nump == 0:
            self._alloc(dim)
        elif self.nump + extra > self.maxp:
            self.maxp = max(self.maxp*2, self.maxp+extra)
            self.x.resize((self.maxp, dim))
            self.fx.resize((self.maxp, 1))

    def get_x(self):
        """Get the list of data points

        :return: List of data points
        """
        return self.x[:self.nump, :]

    def get_fx(self):
        """Get the list of function values for the data points.

        :return: List of function values
        """
        return self.fx[:self.nump, :]

    def add_point(self, xx, fx):
        """Add a new function evaluation

        :param xx: Point to add
        :param fx: The function value of the point to add
        """
        dim = len(xx)
        self._realloc(dim)
        self.x[self.nump, :] = xx
        self.fx[self.nump, :] = fx
        self.nump += 1

        if self.k is None:
            self.k = kriging(self.x[:self.nump+1, :],
                             self.fx[:self.nump+1, :], self.maxp)
            self.k.train()

        # add point to kriging model
        self.k.addPoint(xx, fx)
        self.updated = False

    def eval(self, xx, d=None):
        """Evaluate the Kriging interpolant at the point xx

        :param xx: Point where to evaluate
        :return: Value of the Kriging interpolant at x
        """
        if self.updated is False:
            self.k.train()
        self.updated = True

        fx = self.k.predict(xx.ravel())
        return fx

    def evals(self, xx, d=None):
        """Evaluate the Kriging interpolant at the points xx

        :param xx: Points where to evaluate
        :return: Values of the Kriging interpolant at x
        """
        if self.updated is False:
            self.k.train()
        self.updated = True

        length = xx.shape[0]
        fx = np.zeros(shape=(length, 1))
        for i in range(length):
            fx[i, 0] = self.eval(np.asarray(xx[i]))
        return fx

    def deriv(self, x, d=None):
        """Evaluate the derivative of the rbf interpolant at x

        :param x: Data point
        :return: Derivative of the rbf interpolant at x
        """
        # FIXME, To be implemented
        raise NotImplementedError

# ====================================================================


def _main():
    """Main test routine"""

    def test_f(x):
        """Test function"""
        fx = x[1]*np.sin(x[0]) + x[0]*np.cos(x[1])
        return fx

    fhat = KrigingInterpolant(50)
    print("fhat.maxp: %i" % fhat.maxp)

    # Set up more points
    xs = np.random.rand(50, 2)
    for i in range(40):
        xx = xs[i, :]
        fx = test_f(xx)
        fhat.add_point(xx, fx)
    fhx = fhat.evals(xs[:10, :])
    for i in range(10):
        fx = test_f(xs[i, :])
        print("Err: %e" % (np.abs(fx-fhx[i])/np.abs(fx)))


if __name__ == "__main__":
    _main()



#!/usr/bin/env python
import numpy as np
import time

class Ackley:
    #  Details: http://www.cs.unm.edu/~neal.holts/dga/benchmarkFunction/ackley.html
    #  Global optimum: f(0,0,...,0)=0
    def __init__(self, dim=10):
        self.xlow = -15 * np.ones(dim)
        self.xup = 20 * np.ones(dim)
        self.dim = dim
        self.info = str(dim)+"-dimensional Ackley function \n" +\
                             "Global optimum: f(0,0,...,0) = 0"
        self.integer = []
        self.continuous = np.arange(0, dim)

    def objfunction(self, x):
        if len(x) != self.dim:
            raise ValueError('Dimension mismatch')
        n = float(len(x))
        return -20.0 * np.exp(-0.2*np.sqrt(sum(x**2)/n)) - np.exp(sum(np.cos(2.0*np.pi*x))/n)



"""
.. module:: sot_sync_strategies
   :synopsis: Parallel synchronous optimization strategy
.. moduleauthor:: David Bindel <bindel@cornell.edu>,
    David Eriksson <dme65@cornell.edu>

:Module: sot_sync_strategies
:Author: David Bindel <bindel@cornell.edu>,
    David Eriksson <dme65@cornell.edu>

Synchronous strategies for Stochastic RBF
"""

from __future__ import print_function
import numpy as np
import math
import logging
from experimental_design import SymmetricLatinHypercube, LatinHypercube
from sampling_methods import CandidateDYCORS
from poap.strategy import BaseStrategy, RetryStrategy
from rbf_surfaces import CubicRBFSurface
from rbf_interpolant import RBFInterpolant
from utils import *
from rs_capped import *

# Get module-level logger
logger = logging.getLogger(__name__)


class SyncStrategyNoConstraints(BaseStrategy):
    """Parallel synchronous optimization strategy without non-bound constraints.

    This class implements the parallel synchronous SRBF strategy
    described by Regis and Shoemaker.  After the initial experimental
    design (which is embarrassingly parallel), the optimization
    proceeds in phases.  During each phase, we allow nsamples
    simultaneous function evaluations.  We insist that these
    evaluations run to completion -- if one fails for whatever reason,
    we will resubmit it.  Samples are drawn randomly from around the
    current best point, and are sorted according to a merit function
    based on distance to other sample points and predicted function
    values according to the response surface.  After several
    successive significant improvements, we increase the sampling
    radius; after several failures to improve the function value, we
    decrease the sampling radius.  We restart once the sampling radius
    decreases below a threshold.
    """

    def __init__(self, worker_id, data, response_surface, maxeval, nsamples,
                 exp_design=None, sampling_method=None, extra=None):
        """Initialize the optimization strategy.

        :param worker_id: Start ID in a multistart setting
        :param data: Problem parameter data structure
        :param response_surface: Surrogate model object
        :param maxeval: Function evaluation budget
        :param nsamples: Number of simultaneous fevals allowed
        :param exp_design: Experimental design
        :param sampling_method: Sampling method for finding
            points to evaluate
        :param extra: Points to be added to the experimental design
        """

        self.worker_id = worker_id
        self.data = data
        self.fhat = response_surface
        if self.fhat is None:
            self.fhat = RBFInterpolant(surftype=CubicRBFSurface, maxp=maxeval)

        self.maxeval = maxeval
        self.nsamples = nsamples
        self.extra = extra

        # Default to generate sampling points using Symmetric Latin Hypercube
        self.design = exp_design
        if self.design is None:
            if self.data.dim > 50:
                self.design = LatinHypercube(data.dim, data.dim+1)
            else:
                self.design = SymmetricLatinHypercube(data.dim, 2*(data.dim+1))

        self.xrange = np.asarray(data.xup - data.xlow)

        # algorithm parameters
        self.sigma_min = 0.005
        self.sigma_max = 0.2
        self.sigma_init = 0.2

        self.failtol = max(5, data.dim)
        self.succtol = 3

        self.numeval = 0
        self.status = 0
        self.sigma = 0
        self.resubmitter = RetryStrategy()
        self.xbest = None
        self.fbest = np.inf
        self.fbest_old = None

        # Set up search procedures and initialize
        self.sampling = sampling_method
        if self.sampling is None:
            self.sampling = CandidateDYCORS(data)

        self.check_input()

        # Start with first experimental design
        self.sample_initial()

    def check_input(self):
        self.check_common()
        assert not hasattr(self.data, "eval_ineq_constraints"), "Objective function has constraints,\n" \
            "SyncStrategyNoConstraints can't handle constraints"
        assert not hasattr(self.data, "eval_eq_constraints"), "Objective function has constraints,\n" \
            "SyncStrategyNoConstraints can't handle constraints"

    def check_common(self):
        # Check evaluation budget
        if self.extra is None:
            assert self.maxeval >= self.design.npts, \
                "Experimental design is larger than the evaluation budget"
        else:
            assert self.maxeval >= self.design.npts + self.extra.shape[0], \
                "Experimental design + extra points exceeds the evaluation budget"
        # Check dimensionality
        assert self.design.dim == self.data.dim, \
            "Experimental design and optimization problem have different dimensions"
        if self.extra is not None:
            assert self.data.dim == self.extra.shape[1], \
                "Extra point and optimization problem have different dimensions"
        # Check that the optimization problem makes sense
        check_opt_prob(self.data)

    def proj_fun(self, x):
        x = np.atleast_2d(x)
        return round_vars(self.data, x)

    def log_completion(self, record):
        """Record a completed evaluation to the log.

        :param record: Record of the function evaluation
        :param penalty: Penalty for the given point
        """
        xstr = np.array_str(record.params[0], max_line_width=np.inf,
                            precision=5, suppress_small=True)
        if record.feasible:
            logger.info("{} {:.3e} @ {}".format("True", record.value, xstr))
        else:
            logger.info("{} {:.3e} @ {}".format("False", record.value, xstr))

    def adjust_step(self):
        """Adjust the sampling radius sigma.

        After succtol successful steps, we cut the sampling radius;
        after failtol failed steps, we double the sampling radius.

        :ivar Fnew: Best function value in new step
        :ivar fbest: Previous best function evaluation
        """
        # Initialize if this is the first adaptive step
        if self.fbest_old is None:
            self.fbest_old = self.fbest
            return

        # Check if we succeeded at significant improvement
        if self.fbest < self.fbest_old - 1e-3 * math.fabs(self.fbest_old):
            self.status = max(1, self.status + 1)
        else:
            self.status = min(-1, self.status - 1)
        self.fbest_old = self.fbest

        # Check if step needs adjusting
        if self.status <= -self.failtol:
            self.status = 0
            self.sigma /= 2
            logger.info("Reducing sigma")
        if self.status >= self.succtol:
            self.status = 0
            self.sigma = min([2.0 * self.sigma, self.sigma_max])
            logger.info("Increasing sigma")

    def sample_initial(self):
        """Generate and queue an initial experimental design.
        """
        if self.numeval == 0:
            logger.info("=== Start ===")
        else:
            logger.info("=== Restart ===")
        self.fhat.reset()
        self.sigma = self.sigma_init
        self.status = 0
        self.xbest = None
        self.fbest_old = None
        self.fbest = np.inf
        self.fhat.reset()

        start_sample = self.design.generate_points()
        assert start_sample.shape[1] == self.data.dim, \
            "Dimension mismatch between problem and experimental design"
        start_sample = from_unit_box(start_sample, self.data)
        if self.extra is not None:
            start_sample = np.vstack((start_sample, self.extra))

        for j in range(min(start_sample.shape[0], self.maxeval - self.numeval)):
            start_sample[j, :] = self.proj_fun(start_sample[j, :])  # Project onto feasible region
            proposal = self.propose_eval(start_sample[j, :])
            self.resubmitter.rput(proposal)

        self.sampling.init(start_sample, self.fhat, self.maxeval - self.numeval)

    def sample_adapt(self):
        """Generate and queue samples from the search strategy
        """
        self.adjust_step()
        nsamples = min(self.nsamples, self.maxeval - self.numeval)
        new_points = self.sampling.make_points(npts=nsamples, xbest=self.xbest, sigma=self.sigma,
                                               proj_fun=self.proj_fun)
        for i in range(nsamples):
            proposal = self.propose_eval(np.ravel(new_points[i, :]))
            self.resubmitter.rput(proposal)

    def start_batch(self):
        """Generate and queue a new batch of points
        """
        if self.sigma < self.sigma_min:
            self.sample_initial()
        else:
            self.sample_adapt()

    def propose_action(self):
        """Propose an action
        """
        if self.numeval == self.maxeval:
            return self.propose_terminate()
        elif self.resubmitter.num_eval_outstanding == 0:
            self.start_batch()
        return self.resubmitter.get()

    def on_complete(self, record):
        """Handle completed function evaluation.

        When a function evaluation is completed we need to ask the constraint
        handler if the function value should be modified which is the case for
        say a penalty method. We also need to print the information to the
        logfile, update the best value found so far and notify the GUI that
        an evaluation has completed.

        :param record: Evaluation record
        """

        self.numeval += 1
        record.worker_id = self.worker_id
        record.worker_numeval = self.numeval
        record.feasible = True
        self.log_completion(record)
        self.fhat.add_point(record.params[0], record.value)
        if record.value < self.fbest:
            self.xbest = record.params[0]
            self.fbest = record.value


class SyncStrategyPenalty(SyncStrategyNoConstraints):
    """Parallel synchronous optimization strategy with non-bound constraints.

    This is an extension of SyncStrategyNoConstraints that also works with
    bound constraints. We currently only allow inequality constraints, since
    the candidate based methods don't work well with equality constraints.
    We also assume that the constraints are cheap to evaluate, i.e., so that
    it is easy to check if a given point is feasible. More strategies that
    can handle expensive constraints will be added.

    We use a penalty method in the sense that we try to minimize:

    .. math::
        f(x) + \\mu \\sum_j (\\max(0, g_j(x))^2

    where :math:`g_j(x) \\leq 0` are cheap inequality constraints. As a
    measure of promising function values we let all infeasible points have
    the value of the feasible candidate point with the worst function value,
    since large penalties makes it impossible to distinguish between feasible
    points.

    When it comes to the value of :math:`\\mu`, just choose a very large value.


    """

    def __init__(self, worker_id, data, response_surface, maxeval, nsamples,
                 exp_design=None, sampling_method=None, extra=None,
                 penalty=1e6):
        """Initialize the optimization strategy.

        :param worker_id: Start ID in a multistart setting
        :param data: Problem parameter data structure
        :param response_surface: Surrogate model object
        :param maxeval: Function evaluation budget
        :param nsamples: Number of simultaneous fevals allowed
        :param exp_design: Experimental design
        :param search_procedure: Search procedure for finding
            points to evaluate
        :param extra: Points to be added to the experimental design
        :param penalty: Penalty for violating constraints
        """

        # Evals wrapper for penalty method
        def penalty_evals(fhat, xx):
            penalty = self.penalty_fun(xx).T
            vals = fhat.evals(xx)
            if xx.shape[0] > 1:
                ind = (np.where(penalty <= 0.0)[0]).T
                if ind.shape[0] > 1:
                    ind2 = (np.where(penalty > 0.0)[0]).T
                    ind3 = np.argmax(np.squeeze(vals[ind]))
                    vals[ind2] = vals[ind3]
                    return vals
            return vals + penalty

        # Derivs wrapper for penalty method
        def penalty_derivs(fhat, xx):
            x = np.atleast_2d(xx)
            constraints = np.array(self.data.eval_ineq_constraints(x))
            dconstraints = self.data.deriv_ineq_constraints(x)
            constraints[np.where(constraints < 0.0)] = 0.0
            return np.atleast_2d(fhat.deriv(xx)) + \
                2 * self.penalty * np.sum(
                    constraints * np.rollaxis(dconstraints, 2), axis=2).T

        SyncStrategyNoConstraints.__init__(self,  worker_id, data,
                                           RSPenalty(response_surface, penalty_evals, penalty_derivs),
                                           maxeval, nsamples, exp_design,
                                           sampling_method, extra)
        self.penalty = penalty

    def check_input(self):
        self.check_common()
        assert hasattr(self.data, "eval_ineq_constraints"), "Objective function has no inequality constraints"
        assert not hasattr(self.data, "eval_eq_constraints"), "Objective function has equality constraints,\n" \
            "SyncStrategyPenalty can't handle equality constraints"

    def penalty_fun(self, xx):
        """Computes the penalty for constraint violation

        :param xx: Points to compute the penalty for
        :return: Penalty for constraint violations
        """

        vec = np.array(self.data.eval_ineq_constraints(xx))
        vec[np.where(vec < 0.0)] = 0.0
        vec **= 2
        return self.penalty * np.asmatrix(np.sum(vec, axis=1))

    def on_complete(self, record):
        """Handle completed function evaluation.

        When a function evaluation is completed we need to ask the constraint
        handler if the function value should be modified which is the case for
        say a penalty method. We also need to print the information to the
        logfile, update the best value found so far and notify the GUI that
        an evaluation has completed.

        :param record: Evaluation record
        """
        x = np.zeros((1, record.params[0].shape[0]))
        x[0, :] = record.params[0]
        penalty = self.penalty_fun(x)[0, 0]
        if penalty > 0.0:
            record.feasible = False
        else:
            record.feasible = True
        self.log_completion(record)
        self.numeval += 1
        record.worker_id = self.worker_id
        record.worker_numeval = self.numeval
        self.fhat.add_point(record.params[0], record.value)
        # Check if the penalty function is a new best
        if record.value + penalty < self.fbest:
            self.xbest = record.params[0]
            self.fbest = record.value + penalty


class SyncStrategyProjection(SyncStrategyNoConstraints):
    """Parallel synchronous optimization strategy with non-bound constraints.
    It uses a supplied method to project proposed points onto the feasible
    region in order to always evaluate feasible points which is useful in
    situations where it is easy to project onto the feasible region and where
    the objective function is nonsensical for infeasible points.

    This is an extension of SyncStrategyNoConstraints that also works with
    bound constraints.
    """

    def __init__(self, worker_id, data, response_surface, maxeval, nsamples,
                 exp_design=None, sampling_method=None, extra=None,
                 proj_fun=None):
        """Initialize the optimization strategy.

        :param worker_id: Start ID in a multistart setting
        :param data: Problem parameter data structure
        :param response_surface: Surrogate model object
        :param maxeval: Function evaluation budget
        :param nsamples: Number of simultaneous fevals allowed
        :param exp_design: Experimental design
        :param sampling_method: Search procedure for finding
            points to evaluate
        :param extra: Points to be added to the experimental design
        :param proj_fun: Projection operator
        """

        self.projection = proj_fun

        SyncStrategyNoConstraints.__init__(self,  worker_id, data,
                                           response_surface, maxeval,
                                           nsamples, exp_design,
                                           sampling_method, extra)

    def check_input(self):
        self.check_common()
        assert hasattr(self.data, "eval_ineq_constraints") or \
            hasattr(self.data, "eval_eq_constraints"), \
            "Objective function has no constraints"

    def proj_fun(self, x):
        x = np.atleast_2d(x)
        for i in range(x.shape[0]):
            x[i, :] = self.projection(x[i, :])
        return x



"""
.. module:: mars_interpolant
   :synopsis: MARS model interpolation
.. moduleauthor:: Yi Shen <ys623@cornell.edu>

:Module: mars_interpolant
:Author: Yi Shen <ys623@cornell.edu>
"""

import numpy as np
import numpy.linalg as la
from pyearth import Earth


class MARSInterpolant(Earth):
    """Compute and evaluate a MARS interpolant

    :ivar nump: Current number of points
    :ivar maxp: Initial maximum number of points (can grow)
    :ivar x: Interpolation points
    :ivar fx: Function evaluations of interpolation points
    :ivar dim: Number of dimensions
    :ivar model: MARS interpolaion model
    """

    def __init__(self, maxp=100):
        self.nump = 0
        self.maxp = maxp
        self.x = None     # pylint: disable=invalid-name
        self.fx = None
        self.dim = None
        self.model = Earth()
        self.updated = False

    def reset(self):
        """Reset the interpolation."""
        self.nump = 0
        self.x = None
        self.fx = None
        self.updated = False

    def _alloc(self, dim):
        """Allocate storage for x, fx, rhs, and A.

        :param dim: Number of dimensions
        """
        maxp = self.maxp
        self.dim = dim
        self.x = np.zeros((maxp, dim))
        self.fx = np.zeros((maxp, 1))

    def _realloc(self, dim, extra=1):
        """Expand allocation to accommodate more points (if needed)

        :param dim: Number of dimensions
        :param extra: Number of additional points to accommodate
        """
        if self.nump == 0:
            self._alloc(dim)
        elif self.nump+extra > self.maxp:
            self.maxp = max(self.maxp*2, self.maxp+extra)
            self.x.resize((self.maxp, dim))
            self.fx.resize((self.maxp, 1))

    def get_x(self):
        """Get the list of data points

        :return: List of data points
        """
        return self.x[:self.nump, :]

    def get_fx(self):
        """Get the list of function values for the data points.

        :return: List of function values
        """
        return self.fx[:self.nump, :]

    def add_point(self, xx, fx):
        """Add a new function evaluation

        :param xx: Point to add
        :param fx: The function value of the point to add
        """
        dim = len(xx)
        self._realloc(dim)
        self.x[self.nump, :] = xx
        self.fx[self.nump, :] = fx
        self.nump += 1
        self.updated = False

    def eval(self, xx, d=None):
        """Evaluate the MARS interpolant at the point xx

        :param xx: Point where to evaluate
        :return: Value of the MARS interpolant at x
        """
        if self.updated is False:
            self.model.fit(self.x, self.fx)
        self.updated = True

        xx = np.expand_dims(xx, axis=0)
        fx = self.model.predict(xx)
        return fx[0]

    def evals(self, xx, d=None):
        """Evaluate the MARS interpolant at the points xx

        :param xx: Points where to evaluate
        :return: Values of the MARS interpolant at x
        """
        if self.updated is False:
            self.model.fit(self.x, self.fx)
        self.updated = True

        fx = np.zeros(shape=(xx.shape[0], 1))
        fx[:, 0] = self.model.predict(xx)
        return fx

    def deriv(self, x, d=None):
        """Evaluate the derivative of the MARS interpolant at x

        :param x: Data point
        :return: Derivative of the MARS interpolant at x
        """

        if self.updated is False:
            self.model.fit(self.x, self.fx)
        self.updated = True

        x = np.expand_dims(x, axis=0)
        dfx = self.model.predict_deriv(x, variables=None)
        return dfx[0]

# ====================================================================


def _main():
    """Main test routine"""

    def test_f(x):
        """ Test function"""
        fx = x[1]*np.sin(x[0]) + x[0]*np.cos(x[1])
        return fx

    def test_df(x):
        """ Derivative of test function"""
        dfx = np.array([x[1]*np.cos(x[0]) + np.cos(x[1]),
                        np.sin(x[0]) - x[0]*np.sin(x[1])])
        return dfx

    # Set up Earth model
    fhat = MARSInterpolant(20)

    # Set up initial points to train the MARS model
    xs = np.random.rand(15, 2)
    for x in xs:
        fhat.add_point(x, test_f(x))

    x = np.random.rand(10, 2)
    fhx = fhat.evals(x)
    print(" \n------ (fx - fhx)/|fx| ----- ")
    for i in range(10):
        fx = test_f(x[i, :])
        print("Err: %e" % (abs(fx-fhx[i])/abs(fx)))

    print(" \n ------ (fx - fhx)/|fx| , |dfx-dfhx|/|dfx| -----")
    for i in range(10):
        xx = x[i, :]
        fx = test_f(xx)
        dfx = test_df(xx)
        fhx = fhat.eval(xx)
        dfhx = fhat.deriv(xx)
        print("Err (interp): %e : %e" % (abs(fx-fhx)/abs(fx),
                                         la.norm(dfx-dfhx)/la.norm(dfx)))

if __name__ == "__main__":
    _main()



"""
.. module:: rbf_surfaces
   :synopsis: RBF interpolation surfaces
.. moduleauthor:: David Bindel <bindel@cornell.edu>

:Module: rbf_surfaces
:Author: David Bindel <bindel@cornell.edu>
"""


import numpy as np
import scipy.linalg as la
from rbf_base import SimpleRBFSystem
from rbf_base import SimpleRBFSurface


class LinearRBFSystem(SimpleRBFSystem):
    """Linear RBF system"""

    dpoly = 1
    avgdist = False

    def phi(self, r, epsilon=None):
        return r

    def dphi_div_r(self, r, epsilon=None):
        return (r*0 + 1)/r


class CubicRBFSystem(SimpleRBFSystem):
    """Cubic RBF system"""

    dpoly = 2
    avgdist = False

    def phi(self, r, epsilon=None):
        return r ** 3

    def dphi_div_r(self, r, epsilon=None):
        return 3 * r


class QuinticRBFSystem(SimpleRBFSystem):
    """Quintic RBF system"""

    dpoly = 2
    avgdist = False

    def phi(self, r, epsilon=None):
        return r ** 5

    def dphi_div_r(self, r, epsilon=None):
        return 5 * (r ** 3)


class ThinPlateRBFSystem(SimpleRBFSystem):
    """Thin plate RBF system"""

    dpoly = 2
    avgdist = False
    tiny = np.finfo(np.double).tiny

    def phi(self, r, epsilon=None):
        return r * r * np.log(r + self.tiny)

    def dphi_div_r(self, r, epsilon=None):
        return 2 * np.log(r + self.tiny) + 1

"""
class MultiQuadraticRBFSystem(SimpleRBFSystem):
    " Multi-Quadratic RBF "

    dpoly = 2
    avgdist = True

    def phi(self, r, epsilon):
        return np.sqrt((1.0/epsilon * r) ** 2 + 1)

    def dphi_div_r(self, r, epsilon):
        return 1.0/((epsilon ** 2) * np.sqrt(1 + (1.0/epsilon * r) ** 2))


class InverseMultiQuadraticRBFSystem(SimpleRBFSystem):
    " Inverse Multi-Quadratic RBF "

    dpoly = 2
    avgdist = True

    def phi(self, r, epsilon):
        return 1.0/np.sqrt((1.0 / epsilon * r) ** 2 + 1)

    def dphi_div_r(self, r, epsilon):
        return - 1.0/((epsilon ** 2) * (np.sqrt(1 + (1.0/epsilon * r) ** 2)) ** 3)


class GaussianRBFSystem(SimpleRBFSystem):
    " Gaussian RBF "

    dpoly = 2
    avgdist = True

    def phi(self, r, epsilon):
        return np.exp(-(1.0 / epsilon * r) ** 2)

    def dphi_div_r(self, r, epsilon):
        return - (2.0/(epsilon**2)) * np.exp(-(1.0 / epsilon * r) ** 2)
"""


class LinearRBFSurface(SimpleRBFSurface):
    RBFSystem = LinearRBFSystem


class CubicRBFSurface(SimpleRBFSurface):
    RBFSystem = CubicRBFSystem


class QuinticRBFSurface(SimpleRBFSurface):
    RBFSystem = QuinticRBFSystem


class TPSSurface(SimpleRBFSurface):
    RBFSystem = ThinPlateRBFSystem


"""
class MultiQuadSurface(SimpleRBFSurface):
    RBFSystem = MultiQuadraticRBFSystem


class InvMultiQuadSurface(SimpleRBFSurface):
    RBFSystem = InverseMultiQuadraticRBFSystem


class GaussianSurface(SimpleRBFSurface):
    RBFSystem = GaussianRBFSystem
"""

def toyf(x):
    if len(x.shape) == 1:
        return np.cos(2*x[0] + np.cos(3*x[1]))
    else:
        return np.cos(2*x[:, 0] + np.cos(3*x[:, 1]))


def fd_grad(f, x, h=1e-6):
    df = np.zeros(x.shape)
    xp = x.copy()
    for j in range(x.shape[0]):
        xp[j] = x[j]+h
        fp = f(xp)
        xp[j] = x[j]-h
        fm = f(xp)
        xp[j] = x[j]
        df[j] = (fp-fm)/2/h
    return df


def test_evals(RBFSurface=CubicRBFSurface):
    npts = 5
    x = np.random.random((npts, 2))
    fx = toyf(x)
    s = RBFSurface(x, fx)
    sx = s.eval(x)
    relerr = la.norm(sx-fx)/la.norm(fx)
    assert relerr < 1e-8, "Surface inconsistency: {0:.1e}".format(relerr)


def test_surface(RBFSurface=CubicRBFSurface):
    npts = 5
    x = np.random.random((npts, 2))
    y = np.random.random((2,))
    s = RBFSurface(x, toyf(x))
    dg = s.deriv(y)
    dg_fd = fd_grad(s.eval, y)
    relerr = la.norm(dg-dg_fd)/la.norm(dg)
    assert relerr < 1e-8, "Surface inconsistency: {0:.1e}".format(relerr)


def test_seminorm(RBFSurface=CubicRBFSurface):
    npts = 5
    y = np.random.random((2,))
    x0 = np.random.random((npts, 2))
    x1 = np.zeros((npts+1, 2))
    x1[:-1, :] = x0
    x1[-1, :] = y
    s0 = RBFSurface(x0, toyf(x0))
    s1 = RBFSurface(x1, toyf(x1))
    gref = s1.seminorm()-s0.seminorm()
    g = s0.dseminorm(y, s1.fx[-1])[0]
    relerr = np.abs(g-gref)/gref
    assert relerr < 1e-8, "Seminorm inconsistency: {0:.1e}".format(relerr)


def test_seminorm1(RBFSurface=CubicRBFSurface):
    npts = 5
    x = np.random.random((npts, 2))
    y = np.random.random((2,))
    s = RBFSurface(x, toyf(x))
    g, dg = s.dseminorm(y, toyf(y))
    ns0 = s.seminorm()
    s.add_points(y, toyf(y))
    ns1 = s.seminorm()
    gref = ns1-ns0
    relerr = np.abs(g-gref)/gref
    assert relerr < 1e-8, "Seminorm inconsistency: {0:.1e}".format(relerr)


def test_gutmann(RBFSurface=CubicRBFSurface):
    npts = 5
    tau = -1
    x = np.random.random((npts, 2))
    y = np.random.random((2,))
    s = RBFSurface(x, toyf(x))
    g, dg = s.dseminorm(y, tau)
    dg_fd = fd_grad(lambda y: s.dseminorm(y, tau)[0], y)
    relerr = la.norm(dg-dg_fd)/la.norm(dg)
    assert relerr < 1e-8, "Gutmann merit inconsistency: {0:.1e}".format(relerr)
    h, dh = s.diseminorm(y, tau)
    dh_fd = fd_grad(lambda y: s.diseminorm(y, tau)[0], y)
    relerr = la.norm(dh-dh_fd)/la.norm(dh)
    assert relerr < 1e-8, "Gutmann merit inconsistency: {0:.1e}".format(relerr)


if __name__ == "__main__":
    classes = [CubicRBFSurface, TPSSurface, LinearRBFSurface]
    for c in classes:
        test_evals(c)
        test_surface(c)
        test_seminorm(c)
        test_seminorm1(c)
        test_gutmann(c)



#!/usr/bin/python
"""
..module:: GUI.py
  :synopsis: A Graphical User Interface for pySOT
..moduleauthor:: David Eriksson <dme65@cornell.edu>
"""

import sys
from PySide import QtGui, QtCore
import matplotlib.pyplot as plt
import ntpath
import imp
import os
import numpy as np
import importlib
from poap.controller import *
from poap.strategy import *
import time
from sot_sync_strategies import *
from experimental_design import *
from rs_capped import RSCapped
from sampling_methods import *
from ensemble_surrogate import *
from rbf_interpolant import *
from rbf_surfaces import *
from utils import check_opt_prob
try:
    from kriging_interpolant import KrigingInterpolant
except:
    pass
try:
    from mars_interpolant import MARSInterpolant
except:
    pass

import logging
ntpath.basename("a/b/c")

# =========================== Timing ==============================


class TimerThread(QtCore.QThread):
    time_elapsed = QtCore.Signal(int)

    def __init__(self, parent=None):
        super(TimerThread, self).__init__(parent)
        self.time_start = None

    def start(self, time_start):
        self.time_start = time_start

        return super(TimerThread, self).start()

    def run(self):
        while self.parent().isRunning():
            self.time_elapsed.emit(time.time() - self.time_start)
            time.sleep(1)


class MyThread(QtCore.QThread):
    time_elapsed = QtCore.Signal(int)
    run_timer = False

    def __init__(self, parent=None):
        super(MyThread, self).__init__(parent)

        self.timer_thread = TimerThread(self)
        self.timer_thread.time_elapsed.connect(self.time_elapsed.emit)

    def run(self):
        self.timer_thread.start(time.time())

        while self.run_timer:
            time.sleep(1)

# ======================= Dynamic Plot Update =======================


class DynamicUpdate:

    def __init__(self):
        plt.ion()
        self.figure = None
        self.ax = None
        self.pts = None
        self.lines = None
        self.min_x = None
        self.max_x = None

    def on_launch(self, min_x, max_x):
        self.figure, self.ax = plt.subplots()
        self.pts, = self.ax.plot([], [], 'bo')  # Points
        self.lines, = self.ax.plot([], [], 'r-', linewidth=4.0)  # Lines
        self.ax.set_xlim(min_x, max_x)
        self.ax.grid()
        self.min_x = min_x
        self.max_x = max_x
        plt.xlabel('Evaluations')
        plt.ylabel('Function Value')

    def on_running(self, xdata, ydata):
        if not plt.fignum_exists(self.figure.number):
            self.on_launch(self.min_x, self.max_x)

        self.lines.set_xdata(xdata)
        ycummin = np.minimum.accumulate(ydata)
        self.lines.set_ydata(ycummin)
        ymin = np.amin(ycummin)
        ymax = np.amax(ycummin)
        if ymin == ymax:
            ymax += 1
        self.ax.set_ylim(ymin - 0.1*(ymax - ymin), ymax + 0.1*(ymax - ymin))
        self.pts.set_xdata(xdata)
        self.pts.set_ydata(ydata)
        self.ax.relim()
        self.ax.autoscale_view()
        self.figure.canvas.draw()
        self.figure.canvas.flush_events()

# ============================= Helpers =============================


def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


def path_head(path):
    head, tail = ntpath.split(path)
    return head


def get_object(module_name, object_name):
    """return method object or class object given their names"""
    module = importlib.import_module(module_name)
    obj = getattr(module, object_name)
    return obj


def isfloat(value):
    try:
        float(value)
        return True
    except:
        return False

# ========== Manage communication with the strategy and the GUI ============


class Manager(InputStrategy):

    def __init__(self, controller, maxeval, feasible_merit, plot_progress):
        strategy = controller.strategy
        InputStrategy.__init__(self, controller, strategy)
        self.controller.strategy = self
        self.GUI = None
        self.numeval = 0
        self.killed = 0
        self.plot_progress = plot_progress
        self.feasible_merit = feasible_merit
        if self.plot_progress:
            self.dynamic_plot = DynamicUpdate()
            self.dynamic_plot.on_launch(0, maxeval)
            self.xdata = np.arange(0, maxeval)
            self.ydata = np.nan * np.ones(maxeval,)

    def notify(self, msg):
        pass

    def on_complete(self, rec):
        self.numeval += 1
        rec.value = self.feasible_merit(rec)
        self.GUI.update(rec.params[0], rec.value, self.numeval, self.killed)
        print('Evaluation: {0}'.format(self.numeval))
        print('\tParams: {0}'.format(np.array_str(rec.params[0], max_line_width=np.inf,
                                                  precision=5, suppress_small=True)))
        print('\tValue: {0}'.format(rec.value))

        # Plot
        if self.plot_progress:
            self.ydata[self.numeval-1] = self.feasible_merit(rec)
            self.dynamic_plot.on_running(self.xdata[:self.numeval], self.ydata[:self.numeval])

    def on_kill(self, rec):
        self.killed += 1

    def on_terminate(self, rec):
        self.GUI.printMessage("Optimization finished\n", "green")

    def run(self, GUI):
        self.GUI = GUI
        result = self.controller.run(merit=self.feasible_merit)
        if result is None:
            self.GUI.printMessage("No result", "red")
        else:
            self.GUI.printMessage("Best value: " + str(result.value) + "\n", "blue")
            self.GUI.printMessage('Best solution: {0}\n'.format(
                np.array_str(result.params[0], max_line_width=80, precision=5, suppress_small=True)), "blue")

# ================================= GUI ===================================


class myGUI(QtGui.QWidget):

    def __init__(self):
        # Constructor
        super(myGUI, self).__init__()

        self.numeval = 0
        self.numfail = 0
        self.objfun = None
        self.external = False

        # Run params
        self.fbest = np.inf
        self.feasible = True
        self.xbest = None

        self.data = None
        self.nthreads = 4
        self.maxeval = 1000
        self.nsample = 4
        self.penalty = None

        self.exp_des = None
        self.con_hand = None
        self.rs = None

        # Input check
        self.datainp = False
        self.threadinp = True
        self.penaltyinp = False
        self.evalinp = True
        self.siminp = True
        self.inevinp = False
        self.search = None

        # Controller
        self.controller = None
        self.manager = None
        self.myThread = MyThread(self)
        self.myThread.time_elapsed.connect(self.on_myThread_timeElapsed)

        # Title
        self.titlelbl = QtGui.QLabel("Surrogate Optimization Toolbox (pySOT)", self)
        self.titlelbl.move(300, 10)

        # Plot checkbox
        self.plotprogcb = QtGui.QCheckBox("Plot progress", self)
        self.plotprogcb.move(680, 10)
        self.plotprogcb.toggle()

        # Default options
        self.defaultopts = QtGui.QPushButton('Default Options', self)
        self.defaultopts.clicked.connect(self.defaultOpts)
        self.defaultopts.move(10, 5)
        self.defaultopts.resize(140, 20)

        """ Log text area """
        self.log = QtGui.QTextEdit("", self)
        self.log.setReadOnly(True)
        self.log.setLineWrapMode(QtGui.QTextEdit.NoWrap)
        font = self.log.font()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.log.move(200, 420)
        self.log.resize(595, 166)
        self.log.show()
        self.printMessage("Log file initiated at: ./pySOT_GUI.log\n")
        self.printMessage("Please import your optimization problem\n")

        """ File dialog """
        self.inputDlgBtn = QtGui.QPushButton("Optimization problem", self)
        self.importBtn = QtGui.QPushButton("Import", self)
        self.connect(self.inputDlgBtn, QtCore.SIGNAL("clicked()"), self.openInputDialog)
        self.connect(self.importBtn, QtCore.SIGNAL("clicked()"), self.importAction)
        self.inputline = QtGui.QLineEdit("", self)
        self.inputDlgBtn.move(0, 25)
        self.importBtn.move(710, 25)
        self.inputline.setFixedWidth(540)
        self.inputline.move(170, 30)
        self.inputDlgBtn.show()
        self.importBtn.show()

        """ Info lines """
        self.info1 = QtGui.QLabel(self)
        self.info1.move(570, 65)
        self.info2 = QtGui.QLabel(self)
        self.info2.move(570, 85)
        self.info3 = QtGui.QLabel(self)
        self.info3.move(570, 105)
        self.info4 = QtGui.QLabel(self)
        self.info4.move(570, 125)
        self.info5 = QtGui.QLabel(self)
        self.info5.move(570, 145)

        """ Input lines """

        # Number of threads
        self.threadlbl = QtGui.QLabel("Number of threads", self)
        self.threaderr = QtGui.QLabel(self)
        self.threadline = QtGui.QLineEdit("4", self)
        self.threadline.setFixedWidth(60)
        self.threadline.move(150, 60)
        self.threadlbl.move(5, 65)
        self.threaderr.move(240, 65)
        self.threadline.textChanged[str].connect(self.threadChange)
        self.threadline.show()

        # Number of evaluations
        self.evallbl = QtGui.QLabel("Maximum evaluations", self)
        self.evalerr = QtGui.QLabel(self)
        self.evalline = QtGui.QLineEdit("500", self)
        self.evalline.setFixedWidth(60)
        self.evalline.move(150, 90)
        self.evallbl.move(5, 95)
        self.evalerr.move(240, 95)
        self.evalline.textChanged[str].connect(self.evalChange)
        self.evalline.show()

        # Simultaneous evaluations
        self.simlbl = QtGui.QLabel("Simultaneous evaluations", self)
        self.simerr = QtGui.QLabel(self)
        self.simline = QtGui.QLineEdit("4", self)
        self.simline.setFixedWidth(60)
        self.simline.move(420, 60)
        self.simlbl.move(250, 65)
        self.simerr.move(480, 65)
        self.simline.textChanged[str].connect(self.simChange)
        self.simline.show()

        """ Drop-down lists """

        # Asynchronous
        self.synchlbl = QtGui.QLabel("Asynchronous", self)
        self.synchlist = QtGui.QComboBox(self)
        self.synchlist.addItem("Yes")
        self.synchlist.addItem("No")
        self.synchlist.move(420, 90)
        self.synchlist.setFixedWidth(65)
        self.synchlbl.move(250, 95)
        self.synchlist.show()
        self.synchlist.setCurrentIndex(1)
        self.synchlist.setEnabled(False)

        # Experimental design
        self.explbl = QtGui.QLabel("Experimental design", self)
        self.explist = QtGui.QComboBox(self)
        self.explist.addItem("LatinHypercube")
        self.explist.addItem("SymmetricLatinHypercube")
        self.explist.move(150, 180)
        self.explbl.move(5, 185)
        self.explist.setCurrentIndex(1)
        self.explist.show()
        self.explist.activated[str].connect(self.expActivated)

        # Initial evaluations
        self.inevlbl = QtGui.QLabel("Initial evaluations", self)
        self.ineverr = QtGui.QLabel(self)
        self.inevline = QtGui.QLineEdit("", self)
        self.inevline.setFixedWidth(60)
        self.inevline.move(540, 180)
        self.inevlbl.move(420, 185)
        self.ineverr.move(620, 185)
        self.inevline.textChanged[str].connect(self.inevChange)
        self.inevline.show()

        # Controller
        self.controllerlbl = QtGui.QLabel("Controller", self)
        self.controllerlist = QtGui.QComboBox(self)
        self.controllerlist.addItem("ThreadController")
        self.controllerlist.addItem("SerialController")
        self.controllerlist.move(150, 120)
        self.controllerlbl.move(5, 125)
        self.controllerlist.activated[str].connect(self.controllerChange)
        self.controllerlist.show()

        # Strategy
        self.stratlbl = QtGui.QLabel("Strategy", self)
        self.stratlist = QtGui.QComboBox(self)
        self.stratlist.addItem("SyncStrategyNoConstraints")
        self.stratlist.addItem("SyncStrategyPenalty")
        self.stratlist.move(150, 150)
        self.stratlbl.move(5, 155)
        self.stratlist.activated[str].connect(self.stratChange)
        self.stratlist.show()

        # Penalty
        self.penaltylbl = QtGui.QLabel("Penalty", self)
        self.penaltyline = QtGui.QLineEdit("", self)
        self.penaltyline.setFixedWidth(60)
        self.penaltyline.move(450, 150)
        self.penaltylbl.move(400, 155)
        self.penaltyline.textChanged[str].connect(self.penaltyChange)
        self.penaltyline.setDisabled(True)

        # Search strategy
        self.searchlbl = QtGui.QLabel("Sampling Method", self)
        self.searchlist = QtGui.QComboBox(self)
        self.searchlist.addItem("CandidateDYCORS")
        self.searchlist.addItem("CandidateDDS")
        self.searchlist.addItem("CandidateSRBF")
        self.searchlist.addItem("CandidateUniform")
        self.searchlist.addItem("GeneticAlgorithm")
        self.searchlist.addItem("MultiStartGradient")
        self.searchlist.move(150, 220)
        self.searchlbl.move(5, 225)
        self.searchlist.show()

        # Response surface
        self.rslbl = QtGui.QLabel("Response Surface", self)
        self.rslist = QtGui.QComboBox(self)
        self.rslist.addItem("Cubic RBF")
        self.rslist.addItem("Linear RBF")
        self.rslist.addItem("Thin-Plate RBF")
        # Check for Kriging support
        try:
            krig = KrigingInterpolant()
            self.rslist.addItem("Kriging")
        except:
            self.printMessage("WARNING: pyKriging was not found, Kriging is not supported\n")
        # Check for MARS support
        try:
            mars = MARSInterpolant()
            self.rslist.addItem("MARS")
        except:
            self.printMessage("WARNING: py-earth was not found, MARS is not supported\n")
        self.rslist.activated[str].connect(self.rsActivated)
        self.rslist.move(540, 220)
        self.rslbl.move(420, 225)
        self.rslist.show()

        # Tail for RBF
        self.taillbl = QtGui.QLabel("Tail function", self)
        self.taillist = QtGui.QComboBox(self)
        self.taillist.addItem("LinearTail")
        self.taillist.addItem("ConstantTail")
        self.taillist.move(540, 250)
        self.taillbl.move(420, 255)
        self.taillist.show()

        # Capping
        self.rsclbl = QtGui.QLabel("Wrapper", self)
        self.rsclist = QtGui.QComboBox(self)
        self.rsclist.addItem("None")
        self.rsclist.addItem("RSCapped")
        self.rsclist.addItem("RSUnitBox")
        self.rsclist.move(540, 280)
        self.rsclbl.move(420, 285)
        self.rsclist.setCurrentIndex(0)
        self.rsclist.show()

        # Optimization button
        self.optimizebtn = QtGui.QPushButton('Optimize', self)
        self.optimizebtn.setStyleSheet("background-color: silver")
        self.optimizebtn.clicked.connect(self.optimizeActivated)
        self.optimizebtn.move(10, 420)
        self.optimizebtn.resize(80, 50)

        # Stop button
        self.stopbtn = QtGui.QPushButton('Stop', self)
        self.stopbtn.setStyleSheet("background-color: red")
        self.stopbtn.clicked.connect(self.stopActivated)
        self.stopbtn.move(100, 420)
        self.stopbtn.resize(80, 50)
        self.stopbtn.setEnabled(False)

        """ Run information """

        # Number of Evaluations
        temp = QtGui.QLabel("Completed Evals: ", self)
        temp.move(5, 480)
        self.nevallbl = QtGui.QLabel("", self)
        self.nevallbl.move(120, 480)

        # Number of crashed evaluations
        temp = QtGui.QLabel("Failed Evals: ", self)
        temp.move(5, 500)
        self.faillbl = QtGui.QLabel("", self)
        self.faillbl.move(120, 500)

        # Best value found
        temp = QtGui.QLabel("Best value: ", self)
        temp.move(5, 520)
        self.bestlbl = QtGui.QLabel("", self)
        self.bestlbl.move(120, 520)

        # Best solution feasible
        temp = QtGui.QLabel("Feasible: ", self)
        temp.move(5, 540)
        self.feaslbl = QtGui.QLabel("", self)
        self.feaslbl.move(120, 540)

        # Time
        temp = QtGui.QLabel("Time elapsed: ", self)
        temp.move(5, 560)
        self.timelbl = QtGui.QLabel("Not running", self)
        self.timelbl.move(120, 560)

        """ Add Tables for Search Strategies and Ensemble Surrogates """

        # Search strategies
        self.searchadd = QtGui.QPushButton('Add', self)
        self.searchadd.clicked.connect(self.searchAdd)
        self.searchadd.move(320, 222)
        self.searchadd.resize(70, 20)

        self.searchremove = QtGui.QPushButton('Remove', self)
        self.searchremove.clicked.connect(self.searchRemove)
        self.searchremove.move(320, 245)
        self.searchremove.resize(70, 20)
        self.searchremove.setEnabled(False)

        self.searchup = QtGui.QPushButton('Move\nUp', self)
        self.searchup.clicked.connect(self.searchUp)
        self.searchup.move(315, 277)
        self.searchup.resize(60, 50)
        self.searchup.setEnabled(False)

        self.searchdown = QtGui.QPushButton('Move\nDown', self)
        self.searchdown.clicked.connect(self.searchDown)
        self.searchdown.move(315, 318)
        self.searchdown.resize(60, 50)
        self.searchdown.setEnabled(False)

        self.searchtable = QtGui.QTableWidget(0, 1, self)
        self.searchtable.horizontalHeader().setVisible(False)
        self.searchtable.move(10, 250)
        self.searchtable.resize(300, 152)
        self.searchtable.setColumnWidth(0, 275)
        self.searchtable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.searchtable.show()

        # Surrogates
        self.rsadd = QtGui.QPushButton('Add', self)
        self.rsadd.clicked.connect(self.rsAdd)
        self.rsadd.move(705, 252)
        self.rsadd.resize(50, 20)

        self.rsremove = QtGui.QPushButton('Remove', self)
        self.rsremove.clicked.connect(self.rsRemove)
        self.rsremove.move(685, 282)
        self.rsremove.resize(90, 20)
        self.rsremove.setEnabled(False)

        self.rsup = QtGui.QPushButton('Move\nUp', self)
        self.rsup.clicked.connect(self.rsUp)
        self.rsup.move(720, 310)
        self.rsup.resize(60, 50)
        self.rsup.setEnabled(False)

        self.rsdown = QtGui.QPushButton('Move\nDown', self)
        self.rsdown.clicked.connect(self.rsDown)
        self.rsdown.move(720, 350)
        self.rsdown.resize(60, 50)
        self.rsdown.setEnabled(False)

        self.rstable = QtGui.QTableWidget(0, 1, self)
        self.rstable.horizontalHeader().setVisible(False)
        self.rstable.move(420, 310)
        self.rstable.resize(300, 92)
        self.rstable.setColumnWidth(0, 275)
        self.rstable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.rstable.show()

    def defaultOpts(self):
        self.printMessage("Not yet activated\n", "red")

    def searchAdd(self):
        row = self.searchtable.rowCount()
        self.searchtable.insertRow(row)
        self.searchtable.setItem(row, 0, QtGui.QTableWidgetItem(self.searchlist.currentText()))
        self.searchremove.setEnabled(True)
        if self.searchtable.rowCount() > 1:
            self.searchup.setEnabled(True)
            self.searchdown.setEnabled(True)

    def searchRemove(self):
        if self.searchtable.rowCount() > 0:
            row = self.searchtable.currentRow()
            self.searchtable.removeRow(row)
            if self.searchtable.rowCount() < 2:
                self.searchup.setEnabled(False)
                self.searchdown.setEnabled(False)
                if self.searchtable.rowCount() == 0:
                    self.searchremove.setEnabled(False)

    def searchDown(self):
        row = self.searchtable.currentRow()
        if row < self.searchtable.rowCount()-1:
            self.searchtable.insertRow(row+2)
            self.searchtable.setItem(row+2, 0, self.searchtable.takeItem(row, 0))
            self.searchtable.setCurrentCell(row+2, 0)
            self.searchtable.removeRow(row)

    def searchUp(self):
        row = self.searchtable.currentRow()
        if row > 0:
            self.searchtable.insertRow(row-1)
            self.searchtable.setItem(row-1, 0, self.searchtable.takeItem(row+1, 0))
            self.searchtable.setCurrentCell(row-1, 0)
            self.searchtable.removeRow(row+1)

    def rsAdd(self):
        row = self.rstable.rowCount()
        str = ""
        if self.rslist.currentText() == "Kriging" or self.rslist.currentText() == "MARS":
            str = self.rslist.currentText()
        else:
            str = self.rslist.currentText() + ", "
            str += self.taillist.currentText()
            if self.rsclist.currentText() == "RSCapped":
                str += ", Median Cap"
            elif self.rsclist.currentText() == "RSUnitBox":
                str += ", Unit Box"

        # Check if string already exists
        for r in range(self.rstable.rowCount()):
            if str == self.rstable.item(r, 0).text():
                return

        self.rstable.insertRow(row)
        self.rstable.setItem(row, 0, QtGui.QTableWidgetItem(str))
        self.rsremove.setEnabled(True)
        if self.rstable.rowCount() > 1:
            self.rsup.setEnabled(True)
            self.rsdown.setEnabled(True)

    def rsRemove(self):
        if self.rstable.rowCount() > 0:
            row = self.rstable.currentRow()
            self.rstable.removeRow(row)
            if self.rstable.rowCount() < 2:
                self.rsup.setEnabled(False)
                self.rsdown.setEnabled(False)
                if self.rstable.rowCount() == 0:
                    self.rsremove.setEnabled(False)

    def rsDown(self):
        row = self.rstable.currentRow()
        if row < self.rstable.rowCount()-1:
            self.rstable.insertRow(row+2)
            self.rstable.setItem(row+2, 0, self.rstable.takeItem(row, 0))
            self.rstable.setCurrentCell(row+2, 0)
            self.rstable.removeRow(row)

    def rsUp(self):
        row = self.rstable.currentRow()
        if row > 0:
            self.rstable.insertRow(row-1)
            self.rstable.setItem(row-1, 0, self.rstable.takeItem(row+1, 0))
            self.rstable.setCurrentCell(row-1, 0)
            self.rstable.removeRow(row+1)

    def printMessage(self, text, color="blue"):
        self.log.moveCursor(QtGui.QTextCursor.End)
        self.log.setTextColor(color)
        self.log.insertPlainText(text)
        sb = self.log.verticalScrollBar()
        sb.setValue(sb.maximum())

    """ Triggers """

    def rsActivated(self, text):
        if (text == "Kriging") or (text == "MARS"):
            self.rsclist.setCurrentIndex(1)
            self.rsclist.setEnabled(False)
            self.taillist.setCurrentIndex(0)
            self.taillist.setEnabled(False)
        else:
            self.rsclist.setEnabled(True)
            self.taillist.setEnabled(True)

    def expActivated(self, text):
        self.inevChange(self.inevline.text())

    def controllerChange(self, text):
        if text == "SerialController":
            self.threadline.setText("1")
            self.threadline.setDisabled(True)
            self.simline.setText("1")
            self.simline.setDisabled(True)
        else:
            self.threadline.setDisabled(False)
            self.simline.setDisabled(False)

    def stratChange(self, text):
        if text == "SyncStrategyNoConstraints":
            self.penaltyline.setDisabled(True)
            self.penaltyline.setText("")
        elif text == "SyncStrategyPenalty":
            self.penaltyline.setDisabled(False)
            self.penaltyline.setText("1e6")

    def stopActivated(self):
        self.printMessage("Optimization aborted\n", "red")
        self.manager.terminate()

    def threadChange(self, text):
        if text.isdigit() and int(text) > 0:
            self.threadinp = True
        else:
            self.threadinp = False
        self.simChange(self.simline.text())

    def penaltyChange(self, text):
        if isfloat(text) and float(text) > 0:
            self.penaltyinp = True
        else:
            self.penaltyinp = False

    def evalChange(self, text):
        if text.isdigit() and int(text) > 0:
            self.evalinp = True
        else:
            self.evalinp = False

    def simChange(self, text):
        if not text.isdigit():
            self.simerr.setText("Invalid input!")
            self.simerr.adjustSize()
            self.siminp = False
        elif (not self.threadline.text().isdigit()) or int(text) > int(self.threadline.text()):
            #self.simerr.setText("Must be less than\nnumber of threads!")
            self.simerr.adjustSize()
            self.siminp = False
        else:
            self.simerr.setText("")
            self.simerr.adjustSize()
            self.siminp = True

    def inevChange(self, text):
        if self.data is None:
            self.ineverr.setText("Need data object!")
            self.inevinp = False
        elif text.isdigit() and int(text) > 0:
            if self.explist.currentText() == "LatinHypercube":
                minpts = self.data.dim + 1  # FIXME, for now
            else:
                minpts = 2*self.data.dim + 1  # FIXME, for now
            if int(text) >= minpts:
                self.ineverr.setText("")
                self.inevinp = True
            else:
                self.ineverr.setText("Need at least " + str(minpts)+" evals")
                self.inevinp = False
        else:
            self.ineverr.setText("Invalid input!")
            self.inevinp = False
        self.ineverr.adjustSize()

    def openInputDialog(self):
        path, _ = QtGui.QFileDialog.getOpenFileName(self, "Open File", os.getcwd())
        self.inputline.setText(path)
        self.inputline.adjustSize()

    def importAction(self):
        try:
            sys.path.append(os.path.dirname(self.inputline.text()))
            mod = imp.load_source(os.path.splitext(path_leaf(self.inputline.text()))[0],
                                  self.inputline.text())  # Load module
            class_ = None
            try:
                class_ = getattr(mod, os.path.splitext(path_leaf(self.inputline.text()))[0])
            except:
                self.printMessage("Import failed: Expected class named " +
                                  os.path.splitext(path_leaf(self.inputline.text()))[0] +
                                  " containing the optimization problem\n", "red")
                return None

            self.data = class_()
            check_opt_prob(self.data)
            # Check if the objective function is external
            if not hasattr(self.data, "objfunction"):
                self.external = True
                self.objfun = getattr(mod, "objfunction")
            else:
                self.external = False
                self.objfun = None
            self.printMessage("Import successful\n", "green")
            self.datainp = True
            self.inevChange(self.inevline.text())
            # Print info
            self.info1.setText(str("Problem name: " + os.path.splitext(path_leaf(self.inputline.text()))[0]))
            self.info1.adjustSize()
            self.info2.setText(str(self.data.dim) + " dimensions")
            self.info2.adjustSize()
            self.info3.setText(str(len(self.data.integer)) + " integer variables")
            self.info3.adjustSize()
            self.info4.setText("Non-box constraints: " + str(hasattr(self.data, 'eval_ineq_constraints')))
            self.info4.adjustSize()
            if self.external:
                self.info5.setText("External objective function")
            else:
                self.info5.setText("Python objective function")
            self.info5.adjustSize()

        except Exception, err:
            self.printMessage("Import failed: " + err.message + "\n", "red")
            self.data = None
            self.datainp = False
            self.external = False
            self.objfun = None
            # Reset info
            self.info1.setText("")
            self.info1.adjustSize()
            self.info2.setText("")
            self.info2.adjustSize()
            self.info3.setText("")
            self.info3.adjustSize()
            self.info4.setText("")
            self.info4.adjustSize()
            self.info5.setText("")
            self.info5.adjustSize()

    def update(self, xnew, fnew, numeval, numfail):
        # Process new information
        x = xnew.reshape((1, xnew.shape[0]))

        if fnew < self.fbest:
            self.fbest = fnew
            self.xbest = xnew
            if hasattr(self.data, 'eval_ineq_constraints') and \
                       np.max(self.data.eval_ineq_constraints(x)) > 0.0:
                self.feasible = False
            else:
                self.feasible = True

        self.numeval = numeval
        self.numfail = numfail

        # Update GUI
        self.nevallbl.setText(str(self.numeval))
        self.nevallbl.adjustSize()
        self.bestlbl.setText("{:4.4f}".format(self.fbest))
        self.bestlbl.adjustSize()
        self.feaslbl.setText(str(self.feasible))
        self.feaslbl.adjustSize()
        self.faillbl.setText(str(self.numfail))
        self.faillbl.adjustSize()

        # Force redraw
        QtGui.QApplication.processEvents()

    def paintEvent(self, event):

        qp = QtGui.QPainter()
        qp.begin(self)
        # Data info
        qp.drawLine(530, 60, 530, 170)
        qp.drawLine(530, 60, 795, 60)
        qp.drawLine(795, 60, 795, 170)
        qp.drawLine(530, 170, 795, 170)
        # Run info
        qp.drawLine(2, 475, 2, 585)
        qp.drawLine(2, 475, 195, 475)
        qp.drawLine(195, 475, 195, 585)
        qp.drawLine(2, 585, 195, 585)
        # Search box and RS box
        qp.drawLine(3, 210, 3, 410)
        qp.drawLine(3, 210, 795, 210)
        qp.drawLine(400, 210, 400, 410)
        qp.drawLine(3, 410, 795, 410)
        qp.drawLine(795, 210, 795, 410)
        qp.end()

    @QtCore.Slot()
    def on_timer_activated(self):
        self.myThread.start()

    @QtCore.Slot(int)
    def on_myThread_timeElapsed(self, seconds):
        self.timelbl.setText("{0} seconds".format(seconds))
        self.timelbl.adjustSize()

        # Force redraw
        QtGui.QApplication.processEvents()

# ==============================================================================

    def turnActionsOff(self):
        self.optimizebtn.setEnabled(False)
        self.searchadd.setEnabled(False)
        self.searchremove.setEnabled(False)
        self.searchup.setEnabled(False)
        self.searchdown.setEnabled(False)
        self.rsadd.setEnabled(False)
        self.rsremove.setEnabled(False)
        self.rsup.setEnabled(False)
        self.rsdown.setEnabled(False)
        self.importBtn.setEnabled(False)
        self.inputDlgBtn.setEnabled(False)
        self.inputline.setEnabled(False)
        self.threadline.setEnabled(False)
        self.simline.setEnabled(False)
        self.evalline.setEnabled(False)
        self.synchlist.setEnabled(False)
        self.controllerlist.setEnabled(False)
        self.stratlist.setEnabled(False)
        self.explist.setEnabled(False)
        self.searchlist.setEnabled(False)
        self.inevline.setEnabled(False)
        self.rslist.setEnabled(False)
        self.taillist.setEnabled(False)
        self.rsclist.setEnabled(False)

    def turnActionsOn(self):
        self.stopbtn.setEnabled(False)
        self.optimizebtn.setEnabled(True)
        self.searchadd.setEnabled(True)
        self.searchremove.setEnabled(True)
        if self.searchtable.rowCount() > 1:
            self.searchup.setEnabled(True)
            self.searchdown.setEnabled(True)
        self.rsadd.setEnabled(True)
        self.rsremove.setEnabled(True)
        if self.rstable.rowCount() > 1:
            self.rsup.setEnabled(True)
            self.rsdown.setEnabled(True)
        self.importBtn.setEnabled(True)
        self.inputDlgBtn.setEnabled(True)
        self.inputline.setEnabled(True)
        if not self.controllerlist.currentText() == "SerialController":
            self.threadline.setEnabled(True)
            self.simline.setEnabled(True)
        self.evalline.setEnabled(True)
        self.controllerlist.setEnabled(True)
        self.stratlist.setEnabled(True)
        self.explist.setEnabled(True)
        self.searchlist.setEnabled(True)
        self.inevline.setEnabled(True)
        self.rslist.setEnabled(True)
        if not (self.rslist.currentText() == "Kriging" or self.rslist.currentText() == "MARS"):
            self.taillist.setEnabled(True)
            self.rsclist.setEnabled(True)

    def printProblemInfo(self):
        pass

    def optimizeActivated(self):

        # Are we ready for this?
        self.numeval = 0
        self.numfail = 0
        self.xbest = None
        self.fbest = np.inf
        self.feasible = np.NaN

        # Reset parameters from last solution
        self.nevallbl.setText("")
        self.nevallbl.adjustSize()
        self.bestlbl.setText("")
        self.bestlbl.adjustSize()
        self.feaslbl.setText("")
        self.feaslbl.adjustSize()
        self.faillbl.setText("")
        self.faillbl.adjustSize()

        # Timer
        self.timelbl.setText("Not running")
        self.timelbl.adjustSize()

        if not self.datainp:
            self.printMessage("No Optimization problem imported\n", "red")
        elif not self.threadinp:
            self.printMessage("Incorrect number of threads\n", "red")
        elif not self.evalinp:
            self.printMessage("Incorrect number of evaluations\n", "red")
        elif not self.siminp:
            self.printMessage("Incorrect number of simultaneous evaluations\n", "red")
        elif not self.inevinp:
            self.printMessage("Incorrect number of initial evaluations\n", "red")
        elif self.stratlist.currentText() == "SyncStrategyPenalty" and not self.penaltyinp:
            self.printMessage("Incorrect penalty\n", "red")
        else:
            self.turnActionsOff()

            # Extract parameters
            self.nthreads = int(self.threadline.text())
            self.maxeval = int(self.evalline.text())
            self.nsample = int(self.simline.text())

            # Experimental design
            try:
                exp_design_class = globals()[self.explist.currentText()]
                self.exp_des = exp_design_class(dim=self.data.dim, npts=int(self.inevline.text()))
            except Exception, err:
                self.printMessage("Failed to initialize experimental design: " +
                                  err.message + "\n", "red")
                self.turnActionsOn()
                return

            # Search strategy
            try:
                if self.searchtable.rowCount() == 0:
                    raise AssertionError("No search strategies specified")
                # Try to parse what strategies we are using
                names = []
                for i in range(self.searchtable.rowCount()):
                    names.append(str(self.searchtable.item(i, 0).text()))
                unique_names = list(set(names))
                if len(unique_names) == 1:
                    if len(names) > 1:
                        self.printMessage("Multiple sampling methods added, but only one unique. "
                                          "Initiating one such instance.\n", "blue")
                    search_strategy_class = globals()[unique_names[0]]
                    self.search = search_strategy_class(data=self.data)
                else:
                    id = range(len(unique_names))
                    weights = []
                    search_strategies = []
                    dictionary = dict(zip(unique_names, id))
                    for name in unique_names:
                        search_strategy_class = globals()[name]
                        search_strategies.append((search_strategy_class(data=self.data)))
                    for i in range(len(names)):
                        weights.append(dictionary[names[i]])
                    self.search = MultiSampling(search_strategies, weights)

            except Exception, err:
                self.printMessage("Failed to initialize search strategy: " +
                                  err.message + "\n", "red")
                self.turnActionsOn()
                return

            try:
                # Check if we need Ensembles or not
                if self.rstable.rowCount() == 0:
                    raise AssertionError("No response surface specified")
                else:
                    # We need to construct en ensemble surrogate
                    rs = []
                    for i in range(self.rstable.rowCount()):
                        # Parse the name of the response surface
                        name = self.rstable.item(i, 0).text().strip().split(",")
                        name = [str(x) for x in name]
                        if len(name) == 1 and name[0] == "Kriging":
                            rs.append(KrigingInterpolant(maxp=self.maxeval))
                        elif len(name) == 1 and name[0] == "MARS":
                            rs.append(MARSInterpolant(maxp=self.maxeval))
                        else:
                            # Kernel
                            surf_ = None
                            if name[0] == "Linear RBF":
                                surf_ = LinearRBFSurface
                            elif name[0] == "Cubic RBF":
                                surf_ = CubicRBFSurface
                            elif name[0] == "Thin-Plate RBF":
                                surf_ = TPSSurface

                            # DSB: FIXME
                            # Tail
                            if name[1] == " LinearTail":
                                pass
                            elif name[1] == " ConstantTail":
                                pass

                            print name
                            # Build RBF (with cap if necessary)
                            if len(name) == 3:
                                if name[2] == " Median Cap":
                                    rs.append(
                                        RSCapped(
                                            RBFInterpolant(surftype=surf_,
                                                           maxp=self.maxeval)))
                                elif name[2] == " Unit Box":
                                    rs.append(
                                        RSUnitbox(
                                            RBFInterpolant(surftype=surf_,
                                                           maxp=self.maxeval), self.data))
                            else:
                                rs.append(
                                    RBFInterpolant(surftype=surf_,
                                                   maxp=self.maxeval))

                    # Finally construct the objects
                    if len(rs) == 1:
                        self.rs = rs[0]
                    else:
                        self.rs = EnsembleSurrogate(rs, maxp=self.maxeval)

            except Exception, err:
                self.printMessage("Failed to initialize response surface: " +
                                  err.message + "\n", "red")
                self.turnActionsOn()
                return

            try:
                # Controller
                if self.controllerlist.currentText() == "SerialController":
                    self.controller = SerialController(self.data.objfunction)
                elif self.controllerlist.currentText() == "ThreadController":
                    self.controller = ThreadController()
                # Strategy
                if self.stratlist.currentText() == "SyncStrategyNoConstraints":
                    strat = SyncStrategyNoConstraints(0, self.data, self.rs, self.maxeval,
                                                      self.nsample, self.exp_des, self.search)
                    self.controller.strategy = strat

                elif self.stratlist.currentText() == "SyncStrategyPenalty":
                    self.penalty = float(self.penaltyline.text())
                    strat = SyncStrategyPenalty(0, self.data, self.rs, self.maxeval,
                                                self.nsample, self.exp_des, self.search,
                                                penalty=self.penalty)
                    self.controller.strategy = strat

                self.controller.strategy = CheckWorkerStrategy(self.controller, self.controller.strategy)
                # Threads
                if self.controllerlist.currentText() == "ThreadController":
                    for _ in range(self.nthreads):
                        if self.external:
                            self.controller.launch_worker(self.objfun(self.controller))
                        else:
                            worker = BasicWorkerThread(self.controller, self.data.objfunction)
                            self.controller.launch_worker(worker)
            except Exception, err:
                self.printMessage("Failed to initiate controller/strategy: " +
                                  err.message + "\n", "red")
                self.turnActionsOn()
                return

            # Optimize
            try:
                self.stopbtn.setEnabled(True)

                self.numeval = 0
                self.xbest = None
                self.fbest = np.inf

                if self.stratlist.currentText() == "SyncStrategyPenalty":
                    def feasible_merit(record):
                        xx = np.zeros((1, record.params[0].shape[0]))
                        xx[0, :] = record.params[0]
                        return record.value + strat.penalty_fun(xx)[0, 0]
                else:
                    def feasible_merit(record):
                        return record.value

                self.manager = Manager(self.controller, self.maxeval, feasible_merit, self.plotprogcb.isChecked())

                self.on_timer_activated()
                self.myThread.run_timer = True
                self.printMessage("Optimization initialized\n")
                self.printProblemInfo()  # Print some information to the logfile

                self.manager.run(self)
            except Exception, err:
                self.printMessage("Optimization failed: " +
                                  err.message + "\n", "red")

            self.myThread.run_timer = False
            time.sleep(1)
            self.printMessage("Runtime: " + self.timelbl.text() + "\n", "blue")
            self.timelbl.setText("Not running")
            self.timelbl.adjustSize()
            self.manager = None
            self.turnActionsOn()

            # Force redraw
            QtGui.QApplication.processEvents()


def GUI():
    # Use logger
    logging.basicConfig(filename="./pySOT_GUI.log",
                        level=logging.INFO)
    app = QtGui.QApplication(sys.argv)
    ex = myGUI()
    qr = ex.frameGeometry()
    cp = QtGui.QDesktopWidget().availableGeometry().center()
    qr.moveCenter(cp)
    ex.move(qr.topLeft())
    ex.setFixedSize(800, 590)
    ex.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    GUI()



from setuptools import setup

setup(
    name='pySOT',
    version='0.1.21',
    packages=['pySOT', 'pySOT.test'],
    url='http://pypi.python.org/pypi/pySOT/',
    license='LICENSE.txt',
    author='David Bindel, David Eriksson, Christine Shoemaker',
    author_email='bindel@cornell.edu, dme65@cornell.edu, shoemaker@nus.edu.sg',
    description='Surrogate Optimization Toolbox',
    long_description=open('README.md').read(),
    install_requires=['pyDOE', 'inspyred', 'pyKriging', 'POAP',
                      'py_dempster_shafer', 'subprocess32', 'matlab_wrapper'],
    classifiers=['Programming Language :: Python :: 2.7'],
)



from utils import *
import scipy.spatial as scp


def candidate_merit_weighted_distance(cand, npts=1):
    """Weighted distance merit function for the candidate points based methods
    :param cand: Candidate point object
    :param npts: Number of points selected for evaluation

    :return: Points selected for evaluation
    """

    new_points = np.ones((npts,  cand.data.dim))

    for i in range(npts):
        ii = cand.next_weight
        weight = cand.weights[(ii+len(cand.weights)) % len(cand.weights)]
        merit = weight*cand.fhvals + \
            (1-weight)*(1.0 - unit_rescale(cand.dmerit))

        merit[cand.dmerit < cand.dtol] = np.inf
        jj = np.argmin(merit)
        cand.fhvals[jj] = np.inf
        new_points[i, :] = cand.xcand[jj, :]

        # Update distances and weights
        ds = scp.distance.cdist(cand.xcand, np.atleast_2d(new_points[i, :]))
        cand.dmerit = np.minimum(cand.dmerit, ds)
        cand.next_weight += 1

    return new_points